package com.gxzc.zen.umps.common

import com.gxzc.zen.api.sys.service.ISysUserService
import org.apache.shiro.authc.AuthenticationInfo
import org.apache.shiro.authc.AuthenticationToken
import org.apache.shiro.authc.SimpleAuthenticationInfo
import org.apache.shiro.authc.UnknownAccountException
import org.apache.shiro.authz.AuthorizationInfo
import org.apache.shiro.authz.SimpleAuthorizationInfo
import org.apache.shiro.realm.AuthorizingRealm
import org.apache.shiro.subject.PrincipalCollection
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired

/**
 *
 * @author NorthLan
 * @date 2018/4/23
 * @url https://noahlan.com
 */
class ZenShiroRealm : AuthorizingRealm() {
    companion object {
        private val logger = LoggerFactory.getLogger(ZenShiroRealm::class.java)
    }

    @Autowired
    private lateinit var userService: ISysUserService

    init {
        this.name = "zenShiro"
    }

    /**
     * 登陆验证
     */
    override fun doGetAuthenticationInfo(token: AuthenticationToken): AuthenticationInfo {
        logger.debug("ZenShiroRealm doGetAuthenticationInfo [${token.principal}]")
        val account = token.principal as String

        // 获取用户信息
        val user = userService.getUserByAccountCacheable(account) ?: throw UnknownAccountException()

        // TODO 账号锁定判定

        // TODO 获取权限信息


        return SimpleAuthenticationInfo(user.account, user.password, ZenSimpleByteSource(user.account + user.salt), user.username)
    }

    /**
     * 权限 角色 hasRoles hasPermission
     */
    override fun doGetAuthorizationInfo(principals: PrincipalCollection?): AuthorizationInfo {
        logger.debug("ZenShiroRealm doGetAuthorizationInfo [$principals]")
        return SimpleAuthorizationInfo().apply {
            addRole("testRole")
        }
    }
}